# Generated by Django 4.2.4 on 2023-11-20 13:51

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0009_auto_20231108_0658"),
    ]

    operations = [
        migrations.AddField(
            model_name="djangoprojectbasenotification",
            name="extra_data",
            field=models.JSONField(null=True),
        ),
    ]
