# Generated by Django 4.2.4 on 2023-08-30 11:18

from django.db import migrations, models
import django_project_base.licensing.models


class Migration(migrations.Migration):
    replaces = [
        ("licensing", "0002_remove_licenseaccessuse_comment_and_more"),
        ("licensing", "0003_alter_licenseaccessuse_comment_and_more"),
        ("licensing", "0004_alter_licenseaccessuse_amount"),
    ]

    dependencies = [
        ("licensing", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="licenseaccessuse",
            name="type",
            field=models.IntegerField(
                choices=[
                    (
                        django_project_base.licensing.models.LicenseAccessUse.UseType["USE"],
                        "Used access",
                    )
                ],
                default=django_project_base.licensing.models.LicenseAccessUse.UseType["USE"],
                verbose_name="Type",
            ),
        ),
        migrations.AlterField(
            model_name="licenseaccessuse",
            name="comment",
            field=models.JSONField(null=True, verbose_name="Comment"),
        ),
        migrations.AlterField(
            model_name="licenseaccessuse",
            name="type",
            field=models.IntegerField(
                choices=[
                    (
                        django_project_base.licensing.models.LicenseAccessUse.UseType["USE"],
                        "Used access",
                    ),
                    (
                        django_project_base.licensing.models.LicenseAccessUse.UseType["ADMIN_USE"],
                        "Used access by admin",
                    ),
                ],
                default=django_project_base.licensing.models.LicenseAccessUse.UseType["USE"],
                verbose_name="Type",
            ),
        ),
        migrations.AddField(
            model_name="licenseaccessuse",
            name="amount",
            field=models.FloatField(default=0),
        ),
    ]
