{% extends "email/base.html" %}
{% load i18n %}
{% block content %}
<div style="width: 45em">
    <h3>{% trans "You have been invited to our application. Your credentials are listed below." %}</h3>
    <div>
        <br/>
        <p>{% trans "Your username:" %} {{ username }}</p>
    </div>
    <br/>
    <h4>{% trans "Have a nice day." %}</h4>
</div>
{% endblock %}
