# Generated by Django 4.2.4 on 2023-08-31 10:50

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0003_alter_djangoprojectbasemessage_options"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="djangoprojectbasemessage",
            options={"verbose_name": "Message"},
        ),
    ]
