# Generated by Django 4.2.4 on 2023-08-18 12:58

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="LicenseAccessUse",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "date",
                    models.DateTimeField(default=django.utils.timezone.now, verbose_name="Date"),
                ),
                (
                    "type",
                    models.IntegerField(
                        choices=[(4, "Used access"), (10, "Used access by admin")],
                        default=4,
                        verbose_name="Type",
                    ),
                ),
                ("user_id", models.CharField(db_index=True, max_length=1024)),
                ("content_type_object_id", models.CharField(max_length=1024)),
                (
                    "comment",
                    models.CharField(max_length=255, null=True, verbose_name="Comment"),
                ),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
    ]
