# Generated by Django 4.2.4 on 2023-10-26 06:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0007_auto_20231026_0555"),
    ]

    operations = [
        migrations.AddField(
            model_name="deliveryreport",
            name="auxiliary_notification",
            field=models.UUIDField(null=True, verbose_name="Auxiliary notification"),
        ),
    ]
