# Generated by Django 4.2.4 on 2023-09-01 06:13

from django.db import migrations


def forwards_func(apps, schema_editor):
    apps.get_model("licensing", "LicenseAccessUse").objects.all().update(comment=None)


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        (
            "licensing",
            "0002_remove_licenseaccessuse_comment_and_more_squashed_0004_alter_licenseaccessuse_amount",
        ),
    ]

    operations = [migrations.RunPython(forwards_func, reverse_func)]
