__title__ = "django project base"
# __version__ = "{version-string}"  #TODO: Switch back to version string when merging to main
__version__ = '0.76.21'
__author__ = "Jure Erznožnik"
__email__ = "jure.erznoznik@gmail.com"
__license__ = "BSD 3-Clause"
__copyright__ = "Copyright 2020-present Jure Erznožnik"
default_app_config = "django_project_base.apps.DjangoProjectBaseConfig"

VERSION = __version__
